@@ -81,6 +81,11 @@ if HAS_USER_SITE:
         'data'   : '{userbase}',
         }
 
+    INSTALL_SCHEMES['osx_framework_user'] = {
+        'headers':
+            '{userbase}/include/{implementation_lower}{py_version_short}{abiflags}/{dist_name}',
+    }
+
 # The keys to an installation scheme; if any new types of files are to be
 # installed, be sure to add an entry to every installation scheme above,
 # and to SCHEME_KEYS here.
@@ -445,12 +450,17 @@ class install(Command):
     def finalize_unix(self):
         """Finalizes options for posix platforms."""
         if self.install_base is not None or self.install_platbase is not None:
-            if ((self.install_lib is None and
-                 self.install_purelib is None and
-                 self.install_platlib is None) or
+            incomplete_scheme = (
+                (
+                    self.install_lib is None and
+                    self.install_purelib is None and
+                    self.install_platlib is None
+                ) or
                 self.install_headers is None or
                 self.install_scripts is None or
-                self.install_data is None):
+                self.install_data is None
+            )
+            if incomplete_scheme:
                 raise DistutilsOptionError(
                       "install-base or install-platbase supplied, but "
                       "installation scheme is incomplete")
@@ -471,8 +481,13 @@ class install(Command):
                     raise DistutilsOptionError(
                           "must not supply exec-prefix without prefix")
 
-                self.prefix = os.path.normpath(sys.prefix)
-                self.exec_prefix = os.path.normpath(sys.exec_prefix)
+                # Allow Fedora to add components to the prefix
+                _prefix_addition = getattr(sysconfig, '_prefix_addition', "")
+
+                self.prefix = (
+                    os.path.normpath(sys.prefix) + _prefix_addition)
+                self.exec_prefix = (
+                    os.path.normpath(sys.exec_prefix) + _prefix_addition)
 
             else:
                 if self.exec_prefix is None:
@@ -505,21 +520,30 @@ class install(Command):
                       "I don't know how to install stuff on '%s'" % os.name)
 
     def select_scheme(self, name):
+        os_name, sep, key = name.partition('_')
+        try:
+            resolved = sysconfig.get_preferred_scheme(key)
+        except Exception:
+            resolved = self._pypy_hack(name)
+        return self._select_scheme(resolved)
+
+    def _select_scheme(self, name):
         """Sets the install directories by applying the install schemes."""
         # it's the caller's problem if they supply a bad name!
-        if (hasattr(sys, 'pypy_version_info') and
-                sys.version_info < (3, 8) and
-                not name.endswith(('_user', '_home'))):
-            if os.name == 'nt':
-                name = 'pypy_nt'
-            else:
-                name = 'pypy'
         scheme = _load_schemes()[name]
         for key in SCHEME_KEYS:
             attrname = 'install_' + key
             if getattr(self, attrname) is None:
                 setattr(self, attrname, scheme[key])
 
+    @staticmethod
+    def _pypy_hack(name):
+        PY37 = sys.version_info < (3, 8)
+        old_pypy = hasattr(sys, 'pypy_version_info') and PY37
+        prefix = not name.endswith(('_user', '_home'))
+        pypy_name = 'pypy' + '_nt' * (os.name == 'nt')
+        return pypy_name if old_pypy and prefix else name
+
     def _expand_attrs(self, attrs):
         for attr in attrs:
             val = getattr(self, attr)
